package mypackage;

import java.util.*;

class Graph
{
	private final int MAX_VERTS = 100;
	private Vertex vertexList[]; // list of vertices
	private int adjMat[][];      // adjacency matrix
	private int nVerts;          // current number of vertices
	private StackX theStack;
	private Vector<Vector<Pair>> cycles;
	//------------------------------------------------------------
	public Graph()               // constructor
	{
		vertexList = new Vertex[MAX_VERTS];
                                       // adjacency matrix
		adjMat = new int[MAX_VERTS][MAX_VERTS];
		nVerts = 0;
		for(int y=0; y<MAX_VERTS; y++)      // set adjacency
			for(int x=0; x<MAX_VERTS; x++)   //    matrix to 0
				adjMat[x][y] = 0;
		theStack = new StackX();
		cycles = new Vector<Vector<Pair>> ();
	}  // end constructor
	//------------------------------------------------------------
	public void addVertex(Pair lab)
	{
		vertexList[nVerts++] = new Vertex(lab);
	}
	//------------------------------------------------------------
	public void addEdge(int start, int end)
	{
		adjMat[start][end] = 1;
		adjMat[end][start] = 1;
	}

	public void addEdge(Pair t_start, Pair t_end)
	{
		addEdge(getVertInd(t_start), getVertInd(t_end));
	}
	//------------------------------------------------------------
	public void displayVertex(int v)
	{
		System.out.println(vertexList[v].getLabel());
	}
	//------------------------------------------------------------
	
	//------------------------------------------------------------
	public void dfs()  // depth-first search
	{
		System.out.println("starting dfs");
		hello.printDoubleArray(adjMat);   
											// begin at vertex 0
		vertexList[0].visited();  // mark it
		displayVertex(0);                 // display it
		theStack.push(0);                 // push it
		//int father_ind = 0; 

		while( !theStack.isEmpty() )      // until stack empty,
		{
			// get an unvisited vertex adjacent to stack top
			int v = getAdjUnvisitedVertex( theStack.peek() );
			if(v == -1)                    // if no such vertex,
				theStack.pop();
			else                           // if it exists,
			{
				System.out.println("updating AdjUnvisitedVertex " + v);
				vertexList[v].visited();  // mark it
				vertexList[v].setFather(vertexList[theStack.peek()].getLabel());
				displayVertex(v);                 // display it
				theStack.push(v);                 // push it
			//	father_ind = v;
				System.out.println("updated " + v +" as " + vertexList[v]);
			}
		}  // end while

   // stack is empty, so we're done
		for(int j=0; j<nVerts; j++)          // reset flags
			vertexList[j].notVisited();
	}  // end dfs
	//------------------------------------------------------------
	// returns an unvisited vertex adj to v
	public int getAdjUnvisitedVertex(int v)
	{
		System.out.println("started getAdjUnvisitedVertex with " + v);
		for(int j=0; j<nVerts; j++)
			if(adjMat[v][j]==1)
				if(!vertexList[j].wasVisited())
					return j;
				else
				{				
					System.out.println("finding a cycle between " + v + " and " + j);
					System.out.println("vertex j =  " + vertexList[j]);
					Vector <Pair> c = new Vector<Pair>();
					c.add(vertexList[v].getLabel());
					int node = v;
					int count = 0;
					//System.out.println("j's father = " + vertexList[j].getFather());
					//System.out.println("j's father index = " + getVertInd(vertexList[j].getFather()));
					//System.out.println("v's father = " + vertexList[v].getFather());
					//System.out.println("v's father index = " + getVertInd(vertexList[v].getFather()));
					while (node != j)
					{
						System.out.println("node = " + node);
						node = getVertInd(vertexList[node].getFather());
						c.add(vertexList[node].getLabel());
						if(count == 10)
							System.exit(0);
						count++;
					}
				cycles.add(c);
			}
		return -1;
	}  // end getAdjUnvisitedVertex()
	//------------------------------------------------------------
	
	public Vector<Vector<Pair>> getCyclesContaining(int f)
	{
	//	hello.printDoubleArray(adjMat); 
		Vector<Vector<Pair>> ans = new Vector<Vector<Pair>>();
		Vector<Vector<Integer>> ans_int;
		int i = 0;
		while(vertexList[i] != null)
		{
		//	System.out.println("i = " + i);
		//	System.out.println(vertexList[i].getLabel());
			if(vertexList[i].getLabel().getFirst() == f)
			{
				ans_int =  getCyclesContVert(i);
		//		System.out.println(ans_int);
				Iterator<Vector <Integer>> iter = ans_int.iterator();
				while(iter.hasNext())
				{
					Vector<Integer> vi; 
					vi = iter.next();
					Iterator<Integer> iter_int = vi.iterator();
					Vector<Pair> vp = new Vector<Pair>();
					while(iter_int.hasNext())
						vp.add(vertexList[iter_int.next()].getLabel());
					ans.add(vp);
				}
			}
			i++;
		}
		return ans;
	}
	
	private Vector<Vector<Integer>> getCyclesContVert(int vert)
	{
	//	System.out.println("started getCyclesContVert with " + vert);
		Vector<Vector<Integer>> ans = new Vector<Vector<Integer>>();
		Vector<Vector<Integer>> pending = new Vector<Vector<Integer>>();
	//	Vector<Integer> visited = new Vector<Integer>();
		Vector<Integer> vi = new Vector<Integer>();
		vi.add(vert);
		pending.add(vi);
	//	visited.add(vert);
		while(!pending.isEmpty())
		{
			//visited.add(i);
		//	Iterator<Vector<Integer>> iter = pending.iterator();
		//	Vector<Vector<Integer>> more_pending = new Vector<Vector<Integer>>();
		//	while(iter.hasNext())
		//	{
				Vector<Integer> vi_old = pending.remove(0);
				int last = vi_old.lastElement();
				int j = 0;
				while(vertexList[j] != null)
				{
					//System.out.println("checking arc: " + last + "," + j +" gives: " + adjMat[last][j]);
					if(adjMat[last][j] > 0 && !vi_old.contains(j))
					{
							Vector<Integer> vi_new = new Vector<Integer>(vi_old);
							vi_new.add(j);
							int first = vi_new.get(0);
							if(adjMat[j][first] > 0)
							{
		//						System.out.println("adding cycle: " + vi_new);
								ans.add(new Vector<Integer> (vi_new));
							}
							pending.add(vi_new);
					}
					j++;
				}
			//	pending.addAll(more_pending);
		//	}
		}
		return ans;
	}
	
	
private int getVertInd(Pair trip)
{
	for(int i = 0; i < vertexList.length; i++)
		if (vertexList[i].isLabel(trip))
			return i;
	return -1;
}


}  // end class Graph