package sample;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import rescuecore2.log.Logger;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import sample.com.BaseMessage;

public class AssignmentSolver {
	private String fileName = "none";
	private StandardWorldModel world;
	private EntityID lastCluster;
	private ArrayList<BuildingCluster> clusters;

	public AssignmentSolver(StandardWorldModel world) {
		this.world = world;
	}

	public void init(String[] args) {
		// default: sample

		Params.ASSIGNMENT_MODE = Params.SAMPLE_AGENTS_ASSIGNMENT;
		// Parse arguments
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("sample"))
				Params.ASSIGNMENT_MODE = Params.SAMPLE_AGENTS_ASSIGNMENT;
			else if (args[i].equals("all_target"))
				Params.ASSIGNMENT_MODE = Params.EFPO_ALL_TARGETS_ASSIGNMENT;
			else if (args[i].equals("singlecluster"))
				Params.ASSIGNMENT_MODE = Params.EFPO_SINGLE_CLUSTER_TARGETS_ASSIGNMENT;
			else if (args[i].equals("multicluster"))
				Params.ASSIGNMENT_MODE = Params.EFPO_MULTIPLE_CLUSTER_TARGETS_ASSIGNMENT;
			else if (args[i].equals("hungaran"))
				Params.ASSIGNMENT_MODE = Params.HUNGARIAN_ASSIGNMENT;
			else if (args[i].equals("dsaa")) {
				Params.ASSIGNMENT_MODE = Params.DECENTRAL_DSA_ASSIGNMENT;
				Params.DSA_VARIANT = Params.DSA_A;
			} else if (args[i].equals("dsab")) {
				Params.ASSIGNMENT_MODE = Params.DECENTRAL_DSA_ASSIGNMENT;
				Params.DSA_VARIANT = Params.DSA_B;
			} else if (args[i].equals("ex1_bb")) {
				Params.ASSIGNMENT_MODE = Params.EXERCISE1_BLACK_BOARD;
			} else if (args[i].equals("ex1_cn")) {
				Params.ASSIGNMENT_MODE = Params.EXERCISE1_CONTRACT_NET;
			}
		}
		// Decide file name for output
		if (Params.ASSIGNMENT_MODE == Params.SAMPLE_AGENTS_ASSIGNMENT)
			fileName = "central_assignment_sample_agents.dat";
		else if (Params.ASSIGNMENT_MODE == Params.EFPO_ALL_TARGETS_ASSIGNMENT)
			fileName = "central_assignment_efpo_all_targets.dat";
		else if (Params.ASSIGNMENT_MODE == Params.EFPO_SINGLE_CLUSTER_TARGETS_ASSIGNMENT)
			fileName = "central_assignment_efpo_single_cluster.dat";
		else if (Params.ASSIGNMENT_MODE == Params.EFPO_MULTIPLE_CLUSTER_TARGETS_ASSIGNMENT)
			fileName = "central_assignment_efpo_multi_cluster.dat";
		else if (Params.ASSIGNMENT_MODE == Params.HUNGARIAN_ASSIGNMENT)
			fileName = "central_assignment_hungarian.dat";
		else if (Params.ASSIGNMENT_MODE == Params.DECENTRAL_DSA_ASSIGNMENT) {
			switch (Params.DSA_VARIANT) {
			case Params.DSA_A:
				fileName = "decentral_assignment_dsa_a.dat";
				break;
			case Params.DSA_B:
				fileName = "decentral_assignment_dsa_b.dat";
				break;
			default:
				fileName = "decentral_assignment_dsa_unknown.dat";
				break;
			}
		} else if (Params.ASSIGNMENT_MODE == Params.EXERCISE1_BLACK_BOARD)
			fileName = "exercise1_black_board.dat";
		else if (Params.ASSIGNMENT_MODE == Params.EXERCISE1_CONTRACT_NET)
			fileName = "exercise1_contract_net.dat";

		File f1 = new File(fileName);
		f1.delete();
		System.out.println("ASSIGMENT_MODE: " + Params.ASSIGNMENT_MODE
		        + " log file: " + fileName);

		clusters = new ArrayList<BuildingCluster>();
		lastCluster = new EntityID(-1);
		return;
	}

	public byte[] act(int time, ArrayList<EntityID> agents,
	        ArrayList<Building> targets) {
		if (targets.size() == 0 || agents.size() == 0) {
			Logger.debugColor("No agents or targets for assignment! targets="
			        + targets.size() + " agents:" + agents.size(),
			        Logger.BG_RED);
			return null;
		}

		// Print statistics to file
		if (time == Params.START_EXPERIMENT_TIME - 1) {
			Stats.writeHeader(fileName, agents, targets);
		}
		Stats.writeStatsToFile(fileName, time, targets, world);

		if (Params.ASSIGNMENT_MODE == Params.SAMPLE_AGENTS_ASSIGNMENT) {
			Logger.debugColor("No assignment computation!", Logger.FG_MAGENTA);
			return null;
		} else if (Params.ASSIGNMENT_MODE == Params.EXERCISE1_BLACK_BOARD) {
			ArrayList<Object> targetsO = new ArrayList<Object>(targets.size());
			for (int i = 0; i < targets.size(); i++)
				targetsO.add((Object) targets.get(i));

			Logger.debugColor("Excercise1BlackBoard: Assigning "
			        + agents.size() + " agents to " + targets.size()
			        + " targets", Logger.BG_GREEN);
			Exercise01BlackBoard assignment = new Exercise01BlackBoard();

			double U[][] = ArrayStuff.computeUtilityMatrix(agents, targetsO,
			        world);
			if (U == null) {
				Logger.errC("ERROR: matrix is null!");
				return null;
			}
			double[][] A = assignment.compute(U);
			int[] agentAss = ArrayStuff.assignmentMatrixToAssignment(A, agents,
			        targetsO, world);
			return SimpleProtocol.buildAssignmentMessage(agents, targets,
			        agentAss, true);
		} else if (Params.ASSIGNMENT_MODE == Params.EXERCISE1_CONTRACT_NET) {
			ArrayList<Object> targetsO = new ArrayList<Object>(targets.size());
			for (int i = 0; i < targets.size(); i++)
				targetsO.add((Object) targets.get(i));

			Logger.debugColor("Excercise1ContractNet: Assigning "
			        + agents.size() + " agents to " + targets.size()
			        + " targets", Logger.BG_GREEN);
			Exercise01ContractNet assignment = new Exercise01ContractNet();

			double U[][] = ArrayStuff.computeUtilityMatrix(agents, targetsO,
			        world);
			if (U == null) {
				Logger.errC("ERROR: matrix is null!");
				return null;
			}
			double[][] A = assignment.compute(U);
			int[] agentAss = ArrayStuff.assignmentMatrixToAssignment(A, agents,
			        targetsO, world);
			return SimpleProtocol.buildAssignmentMessage(agents, targets,
			        agentAss, true);
		} else if (Params.ASSIGNMENT_MODE == Params.HUNGARIAN_ASSIGNMENT) {
			ArrayList<Object> targetsO = new ArrayList<Object>(targets.size());
			for (int i = 0; i < targets.size(); i++)
				targetsO.add((Object) targets.get(i));

			Logger.debugColor("Hungarian: Assigning " + agents.size()
			        + " agents to " + targets.size() + " targets",
			        Logger.BG_GREEN);
			CentralAssignmentHungarian assignment = new CentralAssignmentHungarian();
			assignment.init(agents, targetsO);

			double U[][] = ArrayStuff.computeUtilityMatrix(agents, targetsO,
			        world);
			if (U == null) {
				Logger.errC("ERROR: matrix is null!");
				return null;
			}
			double[][] A = assignment.compute(U);
			int[] agentAss = ArrayStuff.assignmentMatrixToAssignment(A, agents,
			        targetsO, world);
			return SimpleProtocol.buildAssignmentMessage(agents, targets,
			        agentAss, true);
		} else if (Params.ASSIGNMENT_MODE == Params.EFPO_ALL_TARGETS_ASSIGNMENT) {
			ArrayList<Object> targetsO = new ArrayList<Object>(targets.size());
			for (int i = 0; i < targets.size(); i++)
				targetsO.add((Object) targets.get(i));
			CentralAssignmentEFPO assignment = new CentralAssignmentEFPO();
			assignment.init(agents, targetsO);

			double U[][] = ArrayStuff.computeUtilityMatrix(agents, targetsO,
			        world);
			if (U == null) {
				Logger.errC("ERROR: matrix is null!");
				return null;
			}
			double[][] A = assignment.compute(U);
			int[] agentAss = ArrayStuff.assignmentMatrixToAssignment(A, agents,
			        targetsO, world);
			return SimpleProtocol.buildAssignmentMessage(agents, targets,
			        agentAss, true);
		} else if (Params.ASSIGNMENT_MODE == Params.EFPO_MULTIPLE_CLUSTER_TARGETS_ASSIGNMENT) {
			BuildingCluster.updateBuildingClusters(targets, clusters, world);
			// Compute matrix for clusters
			ArrayList<Object> clustersO = new ArrayList<Object>(clusters.size());
			for (int i = 0; i < clusters.size(); i++)
				clustersO.add((Object) clusters.get(i));
			CentralAssignmentEFPO assignment = new CentralAssignmentEFPO();
			assignment.init(agents, clustersO);

			double U[][] = ArrayStuff.computeUtilityMatrix(agents, clustersO,
			        world);
			if (U == null) {
				Logger.errC("ERROR: matrix is null!");
				return null;
			}
			double[][] A = assignment.compute(U);
			int[] clusterAss = ArrayStuff.assignmentMatrixToAssignment(A,
			        agents, clustersO, world);

			// Distribute agents into clusters and compute for each the
			// assignment
			for (int i = 0; i < clusters.size(); i++)
				clusters.get(i).agents.clear();

			for (int i = 0; i < agents.size(); i++) {
				if (clusterAss[i] == -1)
					continue;
				else {
					int c = clusterAss[i];
					clusters.get(c).agents.add(agents.get(i));
				}
			}
			byte[] message = new byte[1];
			message[0] = SimpleProtocol.STATION_ASSIGNMENT_MESSAGE;
			for (int i = 0; i < clusters.size(); i++) {
				BuildingCluster cluster = clusters.get(i);
				if (cluster.agents.size() > 0) {
					int[] agentAss = cluster.computeAgentBuildingAssignment();
					// Concatenate assignment with message
					byte[] assignMsg = SimpleProtocol.buildAssignmentMessage(
					        cluster.agents, cluster.getFiryBuildings(),
					        agentAss, false);
					byte[] newM = new byte[message.length + assignMsg.length];
					for (int j = 0; j < message.length; j++)
						newM[j] = message[j];
					for (int j = 0; j < assignMsg.length; j++)
						newM[message.length + j] = assignMsg[j];
					message = newM;
				}
			}
			return message;
		} else if (Params.ASSIGNMENT_MODE == Params.EFPO_SINGLE_CLUSTER_TARGETS_ASSIGNMENT) {
			BuildingCluster.updateBuildingClusters(targets, clusters, world);

			// Select a single cluster
			EntityID selectedCluster = new EntityID(-1);

			// Use old cluster if still existent
			if (Params.STICK_TO_LAST_CLUSTER) {
				if (lastCluster.getValue() != -1) {
					for (int i = 0; i < clusters.size(); i++) {
						if (clusters.get(i).id == lastCluster) {
							selectedCluster = clusters.get(i).id;
							break;
						}
					}
				}
			}

			// Otherwise select best cluster
			if (selectedCluster.getValue() == -1) {
				double best = -Double.MAX_VALUE;
				for (int i = 0; i < clusters.size(); i++) {
					// double score = clusters.get(i).buildings.size() /
					// clusters.get(i).neededAgents;
					double score = clusters.get(i).buildings.size();
					if (score > best) {
						selectedCluster = clusters.get(i).id;
						best = score;
					}
				}
			}
			lastCluster = selectedCluster;

			if (clusters.isEmpty()) {
				Logger.debugColor("Have no cluster, so also no assignment",
				        Logger.BG_YELLOW);
				return null;
			}

			// Find the cluster with ID
			BuildingCluster cluster = clusters.get(0);
			for (int i = 0; i < clusters.size(); i++) {
				if (clusters.get(i).id == selectedCluster) {
					cluster = clusters.get(i);
					break;
				}
			}
			Logger.debugColor("Selected cluster " + selectedCluster + " with "
			        + cluster.buildings.size() + " buildings",
			        Logger.BG_LIGHTBLUE);

			// Put all agents into selected cluster and compute assignment
			// cluster.agents.clear();
			cluster.agents = agents;
			if (cluster.agents.size() <= 0) {
				Logger.debugColor("No agents in cluster!!", Logger.BG_RED);
			}
			int ass[] = cluster.computeAgentBuildingAssignment();

			return SimpleProtocol.buildAssignmentMessage(agents, cluster
			        .getFiryBuildings(), ass, true);
		} else if (Params.ASSIGNMENT_MODE == Params.DECENTRAL_DSA_ASSIGNMENT) {
			Logger.debugColor("DSA: Assigning " + agents.size() + " agents to "
			        + targets.size() + " targets", Logger.BG_GREEN);
			// Compute utility matrix
			ArrayList<Object> targetsO = new ArrayList<Object>(targets.size());
			for (int i = 0; i < targets.size(); i++)
				targetsO.add((Object) targets.get(i));
			double U[][] = ArrayStuff.computeUtilityMatrix(agents, targetsO,
			        world);
			if (U == null) {
				Logger.errC("ERROR: utility matrix is null!");
				return null;
			}

			ArrayList<EntityID> targetsID = new ArrayList<EntityID>(targets
			        .size());
			for (int i = 0; i < targets.size(); i++)
				targetsID.add(targets.get(i).getID());

			// Maintain for each "simulated" agent a dsa object
			ArrayList<DSA> dsas = new ArrayList<DSA>();
			for (int i = 0; i < agents.size(); i++) {
				DSA dsa = new DSA();
				dsa.init(U, agents, targetsID, agents.get(i));
				dsas.add(dsa);
			}

			// Iterate synchronously the negotiation process
			// until termination
			boolean terminated = false;
			int cnt = 0;
			while (!terminated) {
				Logger.debugColor("Round " + cnt++, Logger.FG_LIGHTBLUE);
				ArrayList<BaseMessage> msgs = new ArrayList<BaseMessage>();
				// Let anyone send
				for (DSA next : dsas) {
					BaseMessage msg = next.sendMessage();
					if (msg != null)
						msgs.add(msg);
				}
				// Let anyone receive
				for (DSA next : dsas)
					next.receiveMessages(msgs);
				// Let anyone compute
				terminated = true;
				for (DSA next : dsas)
					terminated &= next.compute();
			}

			// Build assignment matrix
			double[][] A = new double[U.length][U[0].length];
			for (int i = 0; i < U.length; i++) {
				for (int j = 0; j < U[0].length; j++) {
					A[i][j] = 0.0;
				}
			}
			HashMap<EntityID, Integer> targetMap = new HashMap<EntityID, Integer>();
			HashMap<EntityID, Integer> agentMap = new HashMap<EntityID, Integer>();
			for (int i = 0; i < targets.size(); i++)
				targetMap.put(targets.get(i).getID(), new Integer(i));
			for (int i = 0; i < agents.size(); i++)
				agentMap.put(agents.get(i), new Integer(i));
			for (DSA next : dsas) {
				EntityID t = next.getSelectedTarget();
				EntityID a = next.getIdentity();
				int col = targetMap.get(t);
				int row = agentMap.get(a);
				A[row][col] = 1;
			}
			ArrayStuff.printMatrixDouble(A);
			int[] agentAss = ArrayStuff.assignmentMatrixToAssignment(A, agents,
			        targetsO, world);
			return SimpleProtocol.buildAssignmentMessage(agents, targets,
			        agentAss, true);
		} else {
			Logger.errC("Unable to handle assignment mode "
			        + Params.ASSIGNMENT_MODE);
			return null;
		}
	}
}
