package sample;

import static rescuecore2.misc.Handy.objectsToIDs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.List;

import rescuecore2.log.Logger;
import rescuecore2.messages.Command;
import rescuecore2.standard.components.StandardAgent;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.messages.AKSpeak;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;

/**
   A sample center agent.
 */
public class SampleCentre extends StandardAgent<Building> {
    AssignmentSolver centralSolver = null;
    ArrayList<EntityID> agents = new ArrayList<EntityID>();

    @Override
    public String toString() {
	return "Sample centre";
    }

    public SampleCentre() {
    }

    @Override
    protected void think(int time, ChangeSet changed, Collection<Command> heard) {
	if (centralSolver == null && model != null) {
	    centralSolver = new AssignmentSolver(model);
	    String mode = config.getValue("assignment_mode");
	    String args[] = {mode};
	    centralSolver.init(args);
	}

	// Find all buildings that are on fire
	Collection<EntityID> burning = getBurningBuildings();
	Logger.debugColor("Number of known BURNING buildings: " + burning.size(),Logger.BG_LIGHTBLUE);

	if (time == config.getIntValue(kernel.KernelConstants.IGNORE_AGENT_COMMANDS_KEY)) {
	    // Subscribe to station channels
	    sendSubscribe(time, Params.PLATOON_CHANNEL);
	}

	for (Command next : heard) {
	    // Process speak messages
	    if (next instanceof AKSpeak) {
		AKSpeak speak = (AKSpeak)next;
		EntityID senderId = speak.getAgentID();

		// add fire brigade agents
		if (model.getEntity(senderId).getURN() == StandardEntityURN.FIRE_BRIGADE.toString()) {
		    if(!agents.contains(senderId))
			agents.add(senderId);
		    byte content[] = speak.getContent();	
		    processMessageContent(content, senderId.getValue());				
		}
	    }             
	    sendRest(time);
	}

	// Compute assignment
	ArrayList<Building> targets = new ArrayList<Building>(burning.size());
	for (EntityID next : burning) {
	    Building b = (Building) model.getEntity(next);
	    targets.add(b);
	}				
	byte[] message = centralSolver.act(time, agents, targets);

	// Send out assignment
	if (message != null) {
	    int[] intArray = SimpleProtocol.byteArrayToIntArray(message, true); 
	    Logger.debugColor("STATION SENDS AssignmentMessage: ",Logger.BG_WHITE);
	    for (int i=0; i<intArray.length; i++)
		System.out.print(intArray[i] + " ");
	    System.out.println();

	    sendSpeak(time, Params.STATION_CHANNEL, message);		   
	}
    }

    private void processMessageContent(byte content[], int senderId) {	 
	if (content.length == 0)
	    return;
	byte MESSAGE_TYPE = content[0];

	switch (MESSAGE_TYPE) {
	case SimpleProtocol.POS_MESSAGE:
	    int pos = SimpleProtocol.getPosIdFromMessage(content);
	    Logger.debugColor("Station: heard from agent " + senderId + " Pos: "+ pos ,Logger.BG_LIGHTBLUE);
	    break;
	default:
	    Logger.debugColor("Station: cannot parse message of type " + MESSAGE_TYPE + " Message size is " + 
		    content.length, Logger.BG_RED);		        	
	    break;
	}
    }


    @Override
    protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
	return EnumSet.of(StandardEntityURN.FIRE_STATION,
		StandardEntityURN.AMBULANCE_CENTRE,
		StandardEntityURN.POLICE_OFFICE);
    }

    private Collection<EntityID> getBurningBuildings() {
	Collection<StandardEntity> e = model.getEntitiesOfType(StandardEntityURN.BUILDING);
	List<Building> result = new ArrayList<Building>();
	for (StandardEntity next : e) {
	    if (next instanceof Building) {
		Building b = (Building)next;
		if (b.getFieryness()>0 && b.getFieryness()<4) {
		    result.add(b);
		}
	    }
	}
	// Sort by distance
	Collections.sort(result, new DistanceSorter(location(), model));
	return objectsToIDs(result);
    } 
}
